; PAULMON2, a user-friendly 8051 monitor, by Paul Stoffregen
; Please email comments, suggestions, bugs to paul@pjrc.com

; Modified version for the MP3 Player project.

; PAULMON2 is in the public domain. PAULMON2 is distributed in
; the hope that it will be useful, but without any warranty;
; without even the implied warranty of merchantability or fitness
; for a particular purpose. 

.equ	base, 0x0000		;location for PAULMON2
.equ	vector, 0x2000		;location to LJMP interrupt vectors

; These three parameters tell PAULMON2 where the user's memory is
; installed.  "bmem" and "emem" define the space that will be searched
; for program headers, user installed commands, start-up programs, etc.
; "bmem" and "emem" should be use so they exclude memory areas where
; perphreal devices may be mapped, as reading memory from an io chip
; may reconfigure it unexpectedly.  If flash rom is used, "bmem" and "emem"
; should also include the space where the flash rom is mapped.

.equ	pgm, 0x2000		;default location for the user program
.equ	bmem, 0x1000		;where is the beginning of memory
.equ	emem, 0xFFFF		;end of the memory

; To set the baud rate, use this formula
; baud_const = 256 - (crystal / (12 * 16 * baud))

.equ	baud_const, 254		;19200 baud w/ 7.3728 MHz

.equ	line_delay, 6		;num of char times to pause during uploads


.equ	selected_bank_bit, 0x00	;stored at memory location 0x20
.equ	selected_bank_byte, 0x20 ;byte where selected_bank_bit is stored


.equ    scl_pin, 0x90
.equ    sda_pin, 0x91
.equ    reset_pin, 0x92
.equ    hc165_load_pin, 0x93
.equ    unused_pin, 0x94
.equ    xilinx_cclk_pin, 0x95
.equ    hc165_clk_pin, 0x95
.equ    hc165_data_pin, 0x96
.equ    xilinx_din_pin, 0x97
.equ    flash_a16_pin, 0xB4
.equ    shutdown_pin, 0xB5



; Flash ROM parameters.	 If "has_flash" is set to zero, all flash rom
; features are turned off, otherwise "bflash" and "eflash" should specify
; the memory range which is flash rom.	Though AMD doesn't suggest it,
; you may be able to map only a part of the flash rom with your address
; decoder logic (and not use the rest), but you should be careful that
; "bflash" and "eflash" don't include and memory which is NOT flash rom
; so that the erase algorithm won't keep applying erase pulses until it
; finally gives up (which will stress the thin oxide and degrade the
; flash rom's life and reliability).  "erase_pin" allows you to specify
; the bit address for a pin which (if held low) will tell PAULMON2 to
; erase the flash rom chip when it starts up.  This is useful if you
; download programs with the "start-up" headers on them and the code you've
; put in the flash rom crashes!

.equ	has_flash, 1		;set to non-zero value if flash installed
.equ	bflash, 0x2000		;first memory location of Flash ROM
.equ	eflash, 0xFFFF		;last memory location of Flash ROM
.equ	erase_pin, 0		;00 = disable erase pin feature
.equ	dram_disable, 0xFFC0	;write to here disables writes to DRAM
.equ	dram_enable, 0xFFC1	;write to here enables writes to DRAM

; mp3 player address bus wiring:
;Flash: A15 A14 A13 A12 A11 A10  A9  A8  A7  A6  A5  A4  A3  A2  A1  A0
;8051:  A14 A13 A15 A12 A11  A9  A8 A10  A4  A2  A6  A5  A7  A0  A1  A3
;
;0x5555  0   1   0   1   0   1   0   1   0   1   0   1   0   1   0   1
;0x362D  0   0   1   1   0   1   1   0   0   0   1   0   1   1   0   1 
;
;0x2AAA  0   0   1   0   1   0   1   0   1   0   1   0   1   0   1   0
;0x89D2  1   0   0   0   1   0   0   1   1   1   0   1   0   0   1   0   

; mp3 player data bus wiring:

;Flash:  D7  D6  D5  D4  D3  D2  D1  D0
;8051:   D6  D4  D5  D2  D7  D0  D1  D3

;0xAA     1   0   1   0   1   0   1   0
;0xE2     1   1   1   0   0   0   1   0

;0x55     0   1   0   1   0   1   0   1
;0x1D     0   0   0   1   1   1   0   1

;0xA0     1   0   1   0   0   0   0   0
;0x60     0   1   1   0   0   0   0   0

;0x80     1   0   0   0   0   0   0   0
;0x40     0   1   0   0   0   0   0   0

;0x10     0   0   0   1   0   0   0   0
;0x04     0   0   0   0   0   1   0   0




;sets the base address to add to the flash memory register addresses.
.equ    flash_base, 0

;.equ   flash_en1_addr, 0x5555 + flash_base
.equ    flash_en1_addr, 0x362D + flash_base
;.equ   flash_en1_data, 0xAA
.equ    flash_en1_data, 0xE2

;.equ   flash_en2_addr, 0x2AAA + flash_base
.equ    flash_en2_addr, 0x89D2 + flash_base
;.equ   flash_en2_data, 0x55
.equ    flash_en2_data, 0x1D

;.equ   flash_wr_addr, 0x5555 + flash_base
.equ    flash_wr_addr, 0x362D + flash_base
;.equ   flash_wr_data, 0xA0
.equ    flash_wr_data, 0x60

;.equ   flash_er1_addr, 0x5555 + flash_base
.equ    flash_er1_addr, 0x362D + flash_base
;.equ   flash_er1_data, 0x80
.equ    flash_er1_data, 0x40

;.equ   flash_er2_addr, 0x5555 + flash_base
.equ    flash_er2_addr, 0x362D + flash_base
;.equ   flash_er2_data, 0x10
.equ    flash_er2_data, 0x04


; Please note... much of the memory management code only looks at the
; upper 8 bits of an address, so it's not a good idea to somehow map
; your memory chips (with complex address decoding logic) into chunks
; less than 256 bytes.	In other words, only using a piece of a flash
; rom chip and mapping it between C43A to F91B would confuse PAULMON2
; (as well as require quit a bit of address decoding logic circuitry)

; Several people didn't like the key definations in PAULMON1.
; Actually, I didn't like 'em either, but I never took the time
; to change it.	 Eventually I got used to them, but now it's
; really easy to change which keys do what in PAULMON2.	 You
; can guess what to do below, but don't use lowercase.

.equ	help_key, '?'		;help screen
.equ	dir_key,  'M'		;directory
.equ	run_key,  'R'		;run program
.equ	dnld_key, 'D'		;download
.equ	upld_key, 'U'		;upload
.equ	nloc_key, 'N'		;new memory location
.equ	jump_key, 'J'		;jump to memory location
.equ	dump_key, 'H'		;hex dump memory
.equ	intm_key, 'I'		;hex dump internal memory
.equ	edit_key, 'E'		;edit memory
.equ	clrm_key, 'C'		;clear memory
.equ	erfr_key, 'Z'		;erase flash rom

; timing parameters for AMD Flash ROM 28F256.  These parameters
; and pretty conservative and they seem to work with crystals
; between 6 MHz to 24 MHz... (tested with AMD 28F256 chips only)
; unless you know this is a problem, it is probably not a good
; idea to fiddle with these.

;.equ	pgmwait, 10		;22.1184 MHz crystal assumed
.equ	pgmwait, 19		;11.0592 MHz
.equ	verwait, 5
;.equ	erwait1, 40		;fourty delays @22.1184
.equ	erwait1, 20		;twenty delays for 11.0592 MHz
.equ	erwait2, 229		;each delay .5 ms @22.1184MHz



; These symbols configure paulmon2's internal memory usage.
; It is usually not a good idea to change these unless you
; know that you really have to.

.equ	psw_init, 0		;value for psw (which reg bank to use)
.equ	dnld_parm, 0x10		;block of 16 bytes for download
.equ	stack, 0x80		;location of the stack

;---------------------------------------------------------;
;							  ;
;		     Interrupt Vectors			  ;
;  (and little bits of code crammed in the empty spaces)  ;
;							  ;
;---------------------------------------------------------;

	.org	base
	ljmp	poweron		;reset vector

	.org	base+3
	ljmp	vector+3	;ext int0 vector

r6r7todptr:
	mov	dpl, r6
	mov	dph, r7
	ret

	.org	base+11
	ljmp	vector+11	;timer0 vector

dptrtor6r7:
	mov	r6, dpl
	mov	r7, dph
	ret

	.org	base+19
	ljmp	vector+19	;ext int1 vector

dash:	mov	a, #'-'		;seems kinda trivial, but each time
	ajmp	cout		;this appears in code, it takes 4
	nop			;bytes, but an acall takes only 2

	.org	base+27
	ljmp	vector+27	;timer1 vector

cout_sp:acall	cout
	ajmp	space
	nop

	.org	base+35
	ljmp	vector+35	;uart vector

dash_sp:acall	dash
	ajmp	space
	nop

	.org	base+43
	ljmp	vector+43	;timer2 vector (8052)


;---------------------------------------------------------;
;							  ;
;	The jump table for user programs to call	  ;
;	      subroutines within PAULMON		  ;
;							  ;
;---------------------------------------------------------;

.org	base+46		;never change this line!!  Other
			;programs depend on these locations
			;to access paulmon2 functions

	ajmp	phex1		;2E
	ajmp	cout		;30
	ajmp	cin		;32
	ajmp	phex		;34
	ajmp	phex16		;36
	ajmp	pstr		;38
	ajmp	ghex		;3A
	ajmp	ghex16		;3C
	ajmp	esc		;4E
	ajmp	upper		;40
	nop
	nop
	nop
pcstr_h:ljmp	pcstr		;45
	ajmp	newline		;48
	ljmp	lenstr		;4A
	ljmp	pint8u		;4D
	ljmp	pint8		;50
	ljmp	pint16u		;53
	ljmp	smart_wr	;56
	ljmp	prgm		;59
	ljmp	erall		;5C
	ljmp	find		;5F
cin_filter_h:
	ljmp	cin_filter	;62
	ajmp	asc2hex		;65
	ljmp	erblock		;67
	ljmp	rst_devices	;6A
	ljmp	warm_boot	;6D

;---------------------------------------------------------;
;							  ;
;	       Subroutines for serial I/O		  ;
;							  ;
;---------------------------------------------------------;


cin:	jnb	ri, cin
	clr	ri
	mov	a, sbuf
	ret

dspace: acall	space
space:	mov	a, #' '
cout:	jnb	ti, cout
	clr	ti		;clr ti before the mov to sbuf!
	mov	sbuf, a
	ret

;clearing ti before reading sbuf takes care of the case where
;interrupts may be enabled... if an interrupt were to happen
;between those two instructions, the serial port will just
;wait a while, but in the other order and the character could
;finish transmitting (during the interrupt routine) and then
;ti would be cleared and never set again by the hardware, causing
;the next call to cout to hang forever!

newline2:			;print two newlines
	acall	newline
newline:push	acc		;print one newline
	mov	a, #13
	acall	cout
	mov	a, #10
	acall	cout
	pop	acc
	ret

	;get 2 digit hex number from serial port
	; c = set if ESC pressed, clear otherwise
	; psw.5 = set if return w/ no input, clear otherwise
ghex:
ghex8:	clr	psw.5
ghex8c:
	acall	cin_filter_h	;get first digit
	acall	upper
	cjne	a, #27, ghex8f
ghex8d: setb	c
	clr	a
	ret
ghex8f: cjne	a, #13, ghex8h
	setb	psw.5
	clr	c
	clr	a
	ret
ghex8h: mov	r2, a
	acall	asc2hex
	jc	ghex8c
	xch	a, r2		;r2 will hold hex value of 1st digit
	acall	cout
ghex8j:
	acall	cin_filter_h	;get second digit
	acall	upper
	cjne	a, #27, ghex8k
	sjmp	ghex8d
ghex8k: cjne	a, #13, ghex8m
	mov	a, r2
	clr	c
	ret
ghex8m: cjne	a, #8, ghex8p
ghex8n: acall	cout
	sjmp	ghex8c
ghex8p: cjne	a, #21, ghex8q
	sjmp	ghex8n
ghex8q: mov	r3, a
	acall	asc2hex
	jc	ghex8j
	xch	a, r3
	acall	cout
	mov	a, r2
	swap	a
	orl	a, r3
	clr	c
	ret




	;carry set if esc pressed
	;psw.5 set if return pressed w/ no input
ghex16:
	mov	r2, #0		;start out with 0
	mov	r3, #0
	mov	r4, #4		;number of digits left
	clr	psw.5

ghex16c:
	acall	cin_filter_h
	acall	upper
	cjne	a, #27, ghex16d
	setb	c		;handle esc key
	clr	a
	mov	dph, a
	mov	dpl, a
	ret
ghex16d:cjne	a, #8, ghex16f
	sjmp	ghex16k
ghex16f:cjne	a, #127, ghex16g  ;handle backspace
ghex16k:cjne	r4, #4, ghex16e	  ;have they entered anything yet?
	sjmp	ghex16c
ghex16e:acall	cout
	acall	ghex16y
	inc	r4
	sjmp	ghex16c
ghex16g:cjne	a, #13, ghex16i	  ;return key
	mov	dph, r3
	mov	dpl, r2
	cjne	r4, #4, ghex16h
	clr	a
	mov	dph, a
	mov	dpl, a
	setb	psw.5
ghex16h:clr	c
	ret
ghex16i:mov	r5, a		  ;keep copy of original keystroke
	acall	asc2hex
	jc	ghex16c
	xch	a, r5
	lcall	cout
	mov	a, r5
	push	acc
	acall	ghex16x
	pop	acc
	add	a, r2
	mov	r2, a
	clr	a
	addc	a, r3
	mov	r3, a
	djnz	r4, ghex16c
	clr	c
	mov	dpl, r2
	mov	dph, r3
	ret

ghex16x:  ;multiply r3-r2 by 16 (shift left by 4)
	mov	a, r3
	swap	a
	anl	a, #11110000b
	mov	r3, a
	mov	a, r2
	swap	a
	anl	a, #00001111b
	orl	a, r3
	mov	r3, a
	mov	a, r2
	swap	a
	anl	a, #11110000b
	mov	r2, a
	ret

ghex16y:  ;divide r3-r2 by 16 (shift right by 4)
	mov	a, r2
	swap	a
	anl	a, #00001111b
	mov	r2, a
	mov	a, r3
	swap	a
	anl	a, #11110000b
	orl	a, r2
	mov	r2, a
	mov	a, r3
	swap	a
	anl	a, #00001111b
	mov	r3, a
	ret


	;carry set if invalid input
asc2hex:
	add	a, #208
	jnc	hex_not
	add	a, #246
	jc	hex_maybe
	add	a, #10
	clr	c
	ret
hex_maybe:
	add	a, #249
	jnc	hex_not
	add	a, #250
	jc	hex_not
	add	a, #16
	clr	c
	ret
hex_not:setb	c
	ret


; Highly code efficient resursive call phex contributed
; by Alexander B. Alexandrov <abalex@cbr.spb.ru>

phex:
phex8:
	acall	phex_b
phex_b:	swap	a		;SWAP A will be twice => A unchanged
phex1:	push	acc
	anl	a, #15
	add	a, #0x90	; acc is 0x9X, where X is hex digit
	da	a		; if A to F, C=1 and lower four bits are 0..5
	addc	a, #0x40
	da	a
	acall	cout
	pop	acc
	ret

; the old code... easier to understand
;	push	acc
;	swap	a
;	anl	a, #15
;	add	a, #246
;	jnc	phex_b
;	add	a, #7
;phex_b:add	a, #58
;	acall	cout
;	pop	acc
;phex1:	push	acc
;	anl	a, #15
;	add	a, #246
;	jnc	phex_c
;	add	a, #7
;phex_c:add	a, #58
;	acall	cout
;	pop	acc
;	ret


phex16:
	push	acc
	mov	a, dph
	acall	phex
	mov	a, dpl
	acall	phex
	pop	acc
	ret


;a not so well documented feature of pstr is that you can print
;multiple consecutive strings without needing to reload dptr
;(which takes 3 bytes of code!)... this is useful for inserting
;numbers or spaces between strings.

pstr:	push	acc
pstr1:	clr	a
	movc	a, @a+dptr
	inc	dptr
	jz	pstr2
	mov	c, acc.7
	anl	a, #0x7F
	acall	cout
	jc	pstr2
	sjmp	pstr1
pstr2:	pop	acc
	ret

;converts the ascii code in Acc to uppercase, if it is lowercase

; Code efficient (saves 6 byes) upper contributed
; by Alexander B. Alexandrov <abalex@cbr.spb.ru>

upper:
	cjne	a, #97, upper2
upper2:	jc	upper4		;end if acc < 97
	cjne	a, #123, upper3
upper3:	jnc	upper4		;end if acc >= 123
	add	a, #224		;convert to uppercase
upper4:	ret



lenstr: mov	r0, #0	  ;returns length of a string in r0
	push	acc
lenstr1:clr	a
	movc	a,@a+dptr
	jz	lenstr2
	mov	c,acc.7
	inc	r0
	Jc	lenstr2
	inc	dptr
	sjmp	lenstr1
lenstr2:pop	acc
	ret


esc:  ;checks to see if <ESC> is waiting on serial port
      ;C=clear if no <ESC>, C=set if <ESC> pressed
      ;buffer is flushed
	push	acc
	clr	c
	jnb	ri,esc2
	mov	a,sbuf
	cjne	a,#27,esc1
	setb	c
esc1:	clr	ri
esc2:	pop	acc
	ret


find_h:	ljmp	find

;---------------------------------------------------------;
;							  ;
;    The 'high-level' stuff to interact with the user	  ;
;							  ;
;---------------------------------------------------------;


menu: ;first we print out the prompt, which isn't as simple
      ;as it may seem, since external code can add to the
      ;prompt, so we've got to find and execute all of 'em.
	mov	dptr, #prompt1	  ;give 'em the first part of prompt
	acall	pcstr_h
	lcall	get_bank
	clr	a
	addc	a, #'0'
	acall	cout
	mov	a, r7
	acall	phex
	mov	a, r6
	acall	phex
	;mov	 dptr, #prompt2
	acall	pstr


	acall	cin_filter_h	;get the input, finally
	cjne	a, #':', menu0
	acall	dnld_now
	sjmp	menu
menu0:	acall	upper

;push return address onto stack so we can just jump to the program
	mov	b, #(menu & 255)  ;we push the return address now,
	push	b		  ;to save code later...
	mov	b, #(menu >> 8)	  ;if bogus input, just ret for
	push	b		  ;another prompt.



;first we'll look through memory for a program header that says
;it's a user installed command which matches what the user pressed

;user installed commands need to avoid changing R6/R7, which holds
;the memory pointer.  The stack pointer can't be changed obviously.
;all the other general purpose registers should be available for
;user commands to alter as they wish.

menux:	mov	b, a		;now search for external commands...
	mov	dptr, #bmem
menux1: acall	find_h
	jnc	menuxend	   ;searched all the commands?
	mov	dpl, #4
	clr	a
	movc	a,@a+dptr
	cjne	a, #254, menux2	 ;only FE is an ext command
	inc	dpl
	clr	a
	movc	a,@a+dptr
	cjne	a, b, menux2	  ;only run if they want it
	acall	space
	mov	dpl, #32
	acall	pstr		   ;print command name
	acall	newline
	mov	dpl, #64
	clr	a
	jmp	@a+dptr		;take a leap of faith and jump to it!
menux2: inc	dph
	mov	a, dph
	cjne	a, #((emem+1) >> 8) & 255, menux1
menuxend:
	mov	a, b


;since we didn't find a user installed command, use the builtin ones

menu1a:
menu1b:	cjne	a, #help_key, menu1c
	mov	dptr, #help_cmd2
	acall	pcstr_h
	ajmp	help
menu1c: cjne	a, #dir_key, menu1d
	mov	dptr, #dir_cmd
	acall	pcstr_h
	ajmp	dir
menu1d: cjne	a, #run_key, menu1e
	mov	dptr, #run_cmd
	acall	pcstr_h
	ajmp	run
menu1e: cjne	a, #dnld_key, menu1f
	mov	dptr, #dnld_cmd
	acall	pcstr_h
	ajmp	dnld
menu1f: cjne	a, #upld_key, menu1g
	mov	dptr, #upld_cmd
	acall	pcstr_h
	ajmp	upld
menu1g: cjne	a, #nloc_key, menu1h
	mov	dptr, #nloc_cmd
	acall	pcstr_h
	ajmp	nloc
menu1h: cjne	a, #jump_key, menu1i
	mov	dptr, #jump_cmd
	acall	pcstr_h
	ajmp	jump
menu1i: cjne	a, #dump_key, menu1j
	mov	dptr, #dump_cmd
	acall	pcstr_h
	ajmp	dump
menu1j: cjne	a, #edit_key, menu1k
	mov	dptr, #edit_cmd
	acall	pcstr_h
	ajmp	edit
menu1k: cjne	a, #clrm_key, menu1l
	mov	dptr, #clrm_cmd
	acall	pcstr_h
	ajmp	clrm
menu1l: cjne	a, #erfr_key, menu1m
	mov	dptr, #erfr_cmd
	acall	pcstr_h
	ajmp	erfr
menu1m: cjne	a, #intm_key, menu1n
	mov	dptr, #intm_cmd
	acall	pcstr_h
	ljmp	intm
menu1n:	cjne	a, #'0', menu1o
	acall	newline
	ljmp	set_bank0
menu1o:	cjne	a, #'1', menu1p
	acall	newline
	ljmp	set_bank1

menu1p:

    ;invalid input, no commands to run...
menu_end:			;at this point, we have not found
	ajmp	newline		;anything to run, so we give up.
				;remember, we pushed menu, so newline
				;will just return to menu.

;..........................................................

;---------------------------------------------------------;

;dnlds1 = "Begin sending Intel HEX format file <ESC> to abort"
;dnlds2 = "Download aborted"
;dnlds3 = "Download completed"


;16 byte parameter table: (eight 16 bit values)
;  *   0 = lines received
;  *   1 = bytes received
;  *   2 = bytes written
;  *   3 = bytes unable to write
;  *   4 = incorrect checksums
;  *   5 = unexpected begin of line
;  *   6 = unexpected hex digits (while waiting for bol)
;  *   7 = unexpected non-hex digits (in middle of a line)


dnld_addr20:
	cjne	r0, #2, dnld_unknown_h	;addr records always 2 bytes
	acall	dnld_ghex
	swap	a
	rrc	a
	lcall	set_bank
	acall	dnld_ghex
	sjmp	dnld_get_cksum

dnld_addr32:
	cjne	r0, #2, dnld_unknown_h	;addr records always 2 bytes
	acall	dnld_ghex
	acall	dnld_ghex
	rrc	a
	lcall	set_bank
	sjmp	dnld_get_cksum

dnld_unknown_h:
	ajmp	dnld_unknown

dnld:
	mov	dptr, #dnlds1		 
	acall	pcstr_h		   ;"begin sending file <ESC> to abort"
	acall	dnld_init

	  ;look for begining of line marker ':'
dnld1:	acall	cin
	cjne	a, #27, dnld2	;Test for escape
	ajmp	dnld_esc

dnld2:	cjne	a, #':', dnld2b
	sjmp	dnld2d
dnld2b:	  ;check to see if it's a hex digit, error if it is
	acall	asc2hex
	jc	dnld1
	mov	r1, #6
	acall	dnld_inc
	sjmp	dnld1

dnld_now: ;entry point for main menu detecting ":" character
	mov	a, #'^'
	acall	cout
	acall	dnld_init

dnld2d:	mov	r1, #0
	acall	dnld_inc

	  ;begin taking in the line of data
dnld3:	;mov	a, #'.'
	;acall	cout
	mov	r4, #0		;r4 will count up checksum
	acall	dnld_ghex
	mov	r0, a		;R0 = # of data bytes
	mov	a, #'.'
	acall	cout
	acall	dnld_ghex
	mov	dph, a		;High byte of load address
	acall	dnld_ghex
	mov	dpl, a		;Low byte of load address
	acall	dnld_ghex	;Record type
	cjne	a, #1, dnld4	;End record?
	sjmp	dnld_end
dnld4:	cjne	a, #2, dnld4b
	sjmp	dnld_addr20
dnld4b:	cjne	a, #4, dnld4c
	sjmp	dnld_addr32
dnld4c:	jnz	dnld_unknown	;is it a unknown record type???
dnld5:	mov	a, r0
	jz	dnld_get_cksum
	acall	dnld_ghex	;Get data byte
	mov	r2, a
	mov	r1, #1
	acall	dnld_inc	;count total data bytes received
	mov	a, r2
	lcall	smart_wr	;c=1 if an error writing
	clr	a
	addc	a, #2
	mov	r1, a
;     2 = bytes written
;     3 = bytes unable to write
	acall	dnld_inc
	inc	dptr
	djnz	r0, dnld5
dnld_get_cksum:
	acall	dnld_ghex	;get checksum
	mov	a, r4
	jz	dnld1		;should always add to zero
dnld_sumerr:
	mov	r1, #4
	acall	dnld_inc	;all we can do it count # of cksum errors
	sjmp	dnld1

dnld_unknown:	;handle unknown line type
	mov	a, r0
	jz	dnld_get_cksum	;skip data if size is zero
dnld_ukn2:
	acall	dnld_ghex	;consume all of unknown data
	djnz	r0, dnld_ukn2
	sjmp	dnld_get_cksum

dnld_end:   ;handles the proper end-of-download marker
	mov	a, r0
	jz	dnld_end_3	;should usually be zero
dnld_end_2:
	acall	dnld_ghex	;consume all of useless data
	djnz	r0, dnld_ukn2
dnld_end_3:
	acall	dnld_ghex	;get the last checksum
	mov	a, r4
	jnz	dnld_sumerr
	acall	dnld_dly
	mov	dptr, #dnlds3
	acall	pcstr_h		   ;"download went ok..."
	;consume any cr or lf character that may have been
	;on the end of the last line
	jnb	ri, dnld_sum
	acall	cin
	sjmp	dnld_sum

dnld_esc:   ;handle esc received in the download stream
	acall	dnld_dly
	mov	dptr, #dnlds2	 
	acall	pcstr_h		   ;"download aborted."
	sjmp	dnld_sum

dnld_dly:   ;a short delay since most terminal emulation programs
	    ;won't be ready to receive anything immediately after
	    ;they've transmitted a file... even on a fast Pentium(tm)
	    ;machine with 16550 uarts!
	mov	r0, #0
dnlddly2:mov	r1, #0
	djnz	r1, *		;roughly 128k cycles, appox 0.1 sec
	djnz	r0, dnlddly2
	ret

dnld_inc:     ;increment parameter specified by R1
	      ;note, values in Acc and R1 are destroyed
	mov	a, r1
	anl	a, #00000111b	;just in case
	rl	a
	add	a, #dnld_parm
	mov	r1, a		;now r1 points to lsb
	inc	@r1
	mov	a, @r1
	jnz	dnldin2
	inc	r1
	inc	@r1
dnldin2:ret

dnld_gp:     ;get parameter, and inc to next one (@r1)
	     ;carry clear if parameter is zero.
	     ;16 bit value returned in dptr
	setb	c
	mov	dpl, @r1
	inc	r1
	mov	dph, @r1
	inc	r1
	mov	a, dpl
	jnz	dnldgp2
	mov	a, dph
	jnz	dnldgp2
	clr	c
dnldgp2:ret



;a spacial version of ghex just for the download.  Does not
;look for carriage return or backspace.	 Handles ESC key by
;poping the return address (I know, nasty, but it saves many
;bytes of code in this 4k ROM) and then jumps to the esc
;key handling.	This ghex doesn't echo characters, and if it
;sees ':', it pops the return and jumps to an error handler
;for ':' in the middle of a line.  Non-hex digits also jump
;to error handlers, depending on which digit.
	  
dnld_ghex:
dnldgh1:acall	cin
	acall	upper
	cjne	a, #27, dnldgh3
dnldgh2:pop	acc
	pop	acc
	sjmp	dnld_esc
dnldgh3:cjne	a, #':', dnldgh5
dnldgh4:mov	r1, #5		;handle unexpected beginning of line
	acall	dnld_inc
	pop	acc
	pop	acc
	ajmp	dnld3		;and now we're on a new line!
dnldgh5:acall	asc2hex
	jnc	dnldgh6
	mov	r1, #7
	acall	dnld_inc
	sjmp	dnldgh1
dnldgh6:mov	r2, a		;keep first digit in r2
dnldgh7:acall	cin
	acall	upper
	cjne	a, #27, dnldgh8
	sjmp	dnldgh2
dnldgh8:cjne	a, #':', dnldgh9
	sjmp	dnldgh4
dnldgh9:acall	asc2hex
	jnc	dnldghA
	mov	r1, #7
	acall	dnld_inc
	sjmp	dnldgh7
dnldghA:xch	a, r2
	swap	a
	orl	a, r2
	mov	r2, a
	add	a, r4		;add into checksum
	mov	r4, a
	mov	a, r2		;return value in acc
	ret

;dnlds4 =  "Summary:"
;dnlds5 =  " lines received"
;dnlds6a = " bytes received"
;dnlds6b = " bytes written"

dnld_sum:    ;print out download summary
	mov	a, r6
	push	acc
	mov	a, r7
	push	acc
	mov	dptr, #dnlds4
	acall	pcstr_h
	mov	r1, #dnld_parm
	mov	r6, #dnlds5 & 255
	mov	r7, #dnlds5 >> 8
	acall	dnld_i0
	mov	r6, #dnlds6a & 255
	mov	r7, #dnlds6a >> 8
	acall	dnld_i0
	mov	r6, #dnlds6b & 255
	mov	r7, #dnlds6b >> 8
	acall	dnld_i0

dnld_err:    ;now print out error summary
	mov	r2, #5
dnlder2:acall	dnld_gp
	jc	dnlder3		;any errors?
	djnz	r2, dnlder2
	 ;no errors, so we print the nice message
	mov	dptr, #dnlds13
	acall	pcstr_h
	sjmp	dlnd_sum_done

dnlder3:  ;there were errors, so now we print 'em
	mov	dptr, #dnlds7
	acall	pcstr_h
	  ;but let's not be nasty... only print if necessary
	mov	r1, #(dnld_parm+6)
	mov	r6, #dnlds8 & 255
	mov	r7, #dnlds8 >> 8
	acall	dnld_item
	mov	r6, #dnlds9 & 255
	mov	r7, #dnlds9 >> 8
	acall	dnld_item
	mov	r6, #dnlds10 & 255
	mov	r7, #dnlds10 >> 8
	acall	dnld_item
	mov	r6, #dnlds11 & 255
	mov	r7, #dnlds11 >> 8
	acall	dnld_item
	mov	r6, #dnlds12 & 255
	mov	r7, #dnlds12 >> 8
	acall	dnld_item
dlnd_sum_done:
	pop	acc
	mov	r7, a
	pop	acc
	mov	r6, a
	ajmp	newline

dnld_item:
	acall	dnld_gp		;error conditions
	jnc	dnld_i3
dnld_i2:acall	space
	lcall	pint16u
	acall	r6r7todptr
	acall	pcstr_h
dnld_i3:ret

dnld_i0:acall	dnld_gp		;non-error conditions
	sjmp	dnld_i2


dnld_init:
	;init all dnld parms to zero.
	mov	r0, #dnld_parm
dnld0:	mov	@r0, #0
	inc	r0
	cjne	r0, #dnld_parm + 16, dnld0
	ret


;dnlds7:  = "Errors:"
;dnlds8:  = " bytes unable to write"
;dnlds9:  = " incorrect checksums"
;dnlds10: = " unexpected begin of line"
;dnlds11: = " unexpected hex digits"
;dnlds12: = " unexpected non-hex digits"
;dnlds13: = "No errors detected"



;---------------------------------------------------------;


jump:
	mov	dptr, #prompt8
	acall	pcstr_h
	acall	r6r7todptr
	acall	phex16
	mov	dptr, #prompt4
	acall	pcstr_h
	acall	ghex16
	jb	psw.5, jump3
	jnc	jump2
	ajmp	abort2
jump2:
	acall	dptrtor6r7
jump3:	acall	newline
	mov	dptr, #runs1
	acall	pcstr_h
	acall	r6r7todptr

jump_doit:  ;jump to user code @dptr (this used by run command also)
	clr	a
	mov	psw, a
	mov	b, a
	mov	r0, #7
jditclr:mov	@r0, a		;clear r7 to r1
	djnz	r0, jditclr	;clear r0
	mov	sp, #8		;start w/ sp=7, like a real reset
	push	acc		;unlike a real reset, push 0000
	push	acc		;in case they end with a RET
	jmp	@a+dptr


;---------------------------------------------------------;

dump:	
	mov	r2, #16		;number of lines to print
	acall	newline2
dump1:
	lcall	get_bank
	clr	a
	addc	a, #'0'
	acall	cout
	acall	r6r7todptr
	acall	phex16		;tell 'em the memory location
	mov	a,#':'
	acall	cout_sp
	mov	r3, #16		;r3 counts # of bytes to print
	acall	r6r7todptr
dump2:	clr	a
	movc	a, @a+dptr
	inc	dptr
	acall	phex		;print each byte in hex
	acall	space
	djnz	r3, dump2
	acall	dspace		;print a couple extra space
	mov	r3, #16
	acall	r6r7todptr
dump3:	clr	a
	movc	a, @a+dptr
	inc	dptr
	anl	a, #01111111b	;avoid unprintable characters
	cjne	a, #127, dump3b
	clr	a		;avoid 127/255 (delete/rubout) char
dump3b: add	a, #224
	jc	dump4
	clr	a		;avoid control characters
dump4:	add	a, #32
	acall	cout
	djnz	r3, dump3
	acall	newline
	acall	line_dly
	acall	dptrtor6r7
	acall	esc
	jc	dump5
	djnz	r2, dump1	;loop back up to print next line
dump5:	ajmp	newline

;---------------------------------------------------------;

edit:	   ;edit external ram...
	mov	dptr, #edits1
	acall	pcstr_h
	acall	r6r7todptr
edit1:	acall	phex16
	mov	a,#':'
	acall	cout_sp
	mov	a,#'('
	acall	cout
	acall	dptrtor6r7
	clr	a
	movc	a, @a+dptr
	acall	phex
	mov	dptr,#prompt10
	acall	pcstr_h
	acall	ghex
	jb	psw.5,edit2
	jc	edit2
	acall	r6r7todptr
	lcall	smart_wr
	acall	newline
	acall	r6r7todptr
	inc	dptr
	acall	dptrtor6r7
	ajmp	edit1
edit2:	mov	dptr,#edits2
	ajmp	pcstr_h

;---------------------------------------------------------;

dir:
	mov	dptr, #prompt9
	acall	pcstr_h
	mov	r0, #21
dir0a:	acall	space
	djnz	r0, dir0a
	;mov	dptr, #prompt9b
	acall	pcstr_h

	mov	dph, #(bmem >> 8)
dir1:	acall	find_h		;find the next program in memory
	jc	dir2
dir_end:ajmp	newline		;we're done if no more found
dir2:
	acall	dspace
	mov	dpl, #32	;print its name
	acall	pstr
	mov	dpl, #32	;how long is the name
	acall	lenstr
	mov	a, #33
	clr	c
	subb	a, r0
	mov	r0, a
	mov	a, #' '		;print the right # of spaces
dir3:	acall	cout
	djnz	r0, dir3
	mov	dpl, #0
	acall	phex16		;print the memory location
	mov	r0, #6
	mov	a, #' '
dir4:	acall	cout
	djnz	r0, dir4
	mov	dpl, #4		;now figure out what type it is
	clr	a
	movc	a, @a+dptr
	mov	r2, dph		;save this, we're inside a search

dir5:	cjne	a, #254, dir5b
	mov	dptr, #type1	;it's an external command
	sjmp	dir7
dir5b:	cjne	a, #253, dir5c
dir5bb: mov	dptr, #type4	;it's a startup routine
	sjmp	dir7
dir5c:	cjne	a, #35, dir5d
	mov	dptr, #type2	;it's an ordinary program
	sjmp	dir7
dir5d:	cjne	a, #249, dir5e
	sjmp	dir5bb
dir5e:
dir6:	mov	dptr, #type5	;who knows what the hell it is

dir7:	acall	pcstr_h		   ;print out the type
	mov	dph, r2		;go back and find the next one
	acall	newline
	mov	a, #(emem >> 8)
	cjne	a, dph, dir8	;did we just print the last one?
	ajmp	dir_end
dir8:	inc	dph
	mov	a, dph
	cjne	a, #((emem+1) >> 8) & 255, dir1
	ajmp	dir_end


;type1=Ext Command
;type4=Startup
;type2=Program
;type5=???

;---------------------------------------------------------;


run:   
	acall	newline2
	mov	r2, #255	;first print the menu, count items
	mov	dptr, #bmem
	dec	dph
run2:	inc	dph
	mov	a, dph
	cjne	a, #((emem+1) >> 8) & 255, run2b
	sjmp	run3
run2b:	acall	find_h
	jnc	run3		;have we found 'em all??
	mov	dpl, #4
	clr	a
	movc	a, @a+dptr
	orl	a, #00000011b
	cpl	a
	jz	run2		;this one doesn't run... find next
	acall	dspace
	inc	r2
	mov	a, #'A'		;print the key to press
	add	a, r2
	acall	cout_sp
	acall	dash_sp
	mov	dpl, #32
	acall	pstr		;and the command name
	acall	newline
	ajmp	run2		;and continue doing this
run3:	cjne	r2, #255, run4	;are there any to run??
	mov	dptr, #prompt5
	ajmp	pcstr_h
run4:	mov	dptr, #prompt3	;ask the big question!
	acall	pcstr_h
	mov	a, #'A'
	acall	cout
	acall	dash
	mov	a, #'A'		;such user friendliness...
	add	a, r2		;even tell 'em the choices
	acall	cout
	mov	dptr, #prompt4
	acall	pcstr_h
	acall	cin_filter_h
	cjne	a, #27, run4aa	;they they hit <ESC>
	ajmp	newline
run4aa: mov	r3, a
	mov	a, #31
	clr	c
	subb	a, r2
	mov	a, r3
	jc	run4a
	acall	upper
run4a:	acall	cout
	mov	r3, a
	acall	newline
	;check to see if it's under 32, if so convert to uppercase
	mov	a, r3
	add	a, #(256 - 'A')
	jnc	run4		;if they typed less than 'A'
	mov	r3, a		;R3 has the number they typed
	mov	a, r2		;A=R2 has the maximum number
	clr	c
	subb	a, r3
	jc	run4		;if they typed over the max
	inc	r3
	mov	dptr, #bmem
	dec	dph
run5:	inc	dph
	mov	a, dph
	cjne	a, #((emem+1) >> 8) & 255, run5b
	sjmp	run8
run5b:	acall	find_h
	jnc	run8		;Shouldn't ever do this jump!
	mov	dpl, #4
	clr	a
	movc	a, @a+dptr
	orl	a, #00000011b
	cpl	a
	jz	run5		;this one doesn't run... find next
	djnz	r3, run5	;count til we find the one they want
	acall	newline
	mov	dpl, #64
	ajmp	jump_doit
run8:	ret

;---------------------------------------------------------;

help:
	mov	dptr, #help1txt
	acall	pcstr_h
	mov	r4, #help_key
	mov	dptr, #help_cmd
	acall	help2
	mov	r4, #dir_key
	;mov	 dptr, #dir_cmd
	acall	help2
	mov	r4, #run_key
	;mov	 dptr, #run_cmd
	acall	help2
	mov	r4, #dnld_key
	;mov	 dptr, #dnld_cmd
	acall	help2
	mov	r4, #upld_key
	;mov	 dptr, #upld_cmd
	acall	help2
	mov	r4, #nloc_key
	;mov	 dptr, #nloc_cmd
	acall	help2
	mov	r4, #jump_key
	;mov	 dptr, #jump_cmd
	acall	help2
	mov	r4, #dump_key
	;mov	 dptr, #dump_cmd
	acall	help2
	mov	r4, #intm_key
	;mov	dptr, #intm_cmd
	acall	help2
	mov	r4, #edit_key
	;mov	 dptr, #edit_cmd
	acall	help2
	mov	r4, #clrm_key
	;mov	 dptr, #clrm_cmd
	acall	help2
	mov	r4, #erfr_key
	;mov	 dptr, #erfr_cmd
	acall	help2
	mov	dptr, #help2txt
	acall	pcstr_h
	mov	dptr, #bmem
help3:	acall	find_h
	jnc	help4
	mov	dpl, #4
	clr	a
	movc	a,@a+dptr
	cjne	a, #254, help3a	   ;only FE is an ext command
	acall	dspace
	inc	dpl
	clr	a
	movc	a,@a+dptr
	acall	cout
	acall	dash_sp
	mov	dpl, #32
	acall	pstr
	acall	newline
help3a: inc	dph
	mov	a, dph
	cjne	a, #((emem+1) >> 8) & 255, help3
help4:	
	ajmp	newline

help2:				;print 11 standard lines
	acall	dspace		;given key in R4 and name in dptr
	mov	a, r4
	acall	cout
	acall	dash_sp
	acall	pcstr_h
	ajmp	newline

;---------------------------------------------------------;

upld:

	acall	get_mem
	;assume we've got the beginning address in r3/r2
	;and the final address in r5/r4 (r4=lsb)...

	;print out what we'll be doing
	mov	dptr, #uplds3
	acall	pcstr_h
	mov	a, r3
	acall	phex
	mov	a, r2
	acall	phex
	;mov	 dptr, #uplds4
	acall	pcstr_h
	mov	a, r5
	acall	phex
	mov	a, r4
	acall	phex
	acall	newline

	;need to adjust end location by 1...
	mov	dph, r5
	mov	dpl, r4
	inc	dptr
	mov	r4, dpl
	mov	r5, dph

	mov	dptr, #prompt7
	acall	pcstr_h
	acall	cin
	cjne	a, #27, upld2e
	ajmp	abort_it
upld2e: acall	newline
	mov	dpl, r2
	mov	dph, r3

upld3:	mov	a, r4		;how many more bytes to output??
	clr	c
	subb	a, dpl
	mov	r2, a
	mov	a, r5
	subb	a, dph
	jnz	upld4		;if >256 left, then do next 16
	mov	a, r2
	jz	upld7		;if we're all done
	anl	a, #11110000b
	jnz	upld4		;if >= 16 left, then do next 16
	sjmp	upld5		;otherwise just finish it off
upld4:	mov	r2, #16
upld5:	mov	a, #':'		;begin the line
	acall	cout
	mov	a, r2
	acall	phex		;output # of data bytes
	acall	phex16		;output memory location
	mov	a, dph
	add	a, dpl
	add	a, r2
	mov	r3, a		;r3 will become checksum
	clr	a
	acall	phex		;output 00 code for data
upld6:	clr	a
	movc	a, @a+dptr
	acall	phex		;output each byte
	add	a, r3
	mov	r3, a
	inc	dptr
	djnz	r2, upld6	;do however many bytes we need
	mov	a, r3
	cpl	a
	inc	a
	acall	phex		;and finally the checksum
	acall	newline
	acall	line_dly
	acall	esc
	jnc	upld3		;keep working if no esc pressed
	sjmp	abort_it
upld7:	mov	a, #':'
	acall	cout
	clr	a
	acall	phex
	acall	phex
	acall	phex
	inc	a
	acall	phex
	mov	a, #255
	acall	phex
upld8:	ajmp	newline2


line_dly: ;a brief delay between line while uploading, so the
	;receiving host can be slow (i.e. most windows software)
	mov	a, r0
	push	acc
	mov	r0, #line_delay*2
line_d2:mov	a, th0		;get baud rate const
line_d3:inc	a
	nop
	nop
	jnz	line_d3
	djnz	r0, line_d2
	pop	acc
	mov	r0, a
	ret

;---------------------------------------------------------;

get_mem:     ;this thing gets the begin and end locations for
	     ;a few commands.  If an esc or enter w/ no input,
	     ;it pops it's own return and returns to the menu
	     ;(nasty programming, but we need tight code for 4k rom)
	acall	newline2
	mov	dptr, #beg_str
	acall	pcstr_h
	acall	ghex16
	jc	pop_it
	jb	psw.5, pop_it
	push	dph
	push	dpl
	acall	newline
	mov	dptr, #end_str
	acall	pcstr_h
	acall	ghex16
	mov	r5, dph
	mov	r4, dpl
	pop	acc
	mov	r2, a
	pop	acc
	mov	r3, a
	jc	pop_it
	jb	psw.5, pop_it
	ajmp	newline

pop_it: pop	acc
	pop	acc
abort_it:
	acall	newline
abort2: mov	dptr, #abort
	ajmp	pcstr_h


clrm:
	acall	get_mem
	mov	dptr, #sure
	acall	pcstr_h
	acall	cin_filter_h
	acall	upper
	cjne	a, #'Y', abort_it
	acall	newline2
     ;now we actually do it

clrm2:	mov	dph, r3
	mov	dpl, r2
clrm3:	clr	a
	lcall	smart_wr
	mov	a, r5
	cjne	a, dph, clrm4
	mov	a, r4
	cjne	a, dpl, clrm4
	ret
clrm4:	inc	dptr
	sjmp	clrm3

;---------------------------------------------------------;

nloc:
	mov	dptr, #prompt6
	acall	pcstr_h
	acall	ghex16
	jc	abort2
	jb	psw.5, abort2
	acall	dptrtor6r7
	ajmp	newline2

;---------------------------------------------------------;

erfr:
	acall	newline2
	mov	dptr, #erfr_cmd
	acall	pcstr_h
	mov	a, #','
	acall	cout_sp
	mov	dptr, #sure
	acall	pcstr_h
	acall	cin_filter_h
	acall	upper
	cjne	a, #'Y', abort_it
	acall	newline2
	lcall	erall
	mov	dptr, #erfr_ok
	jnc	erfr_end
	mov	dptr, #erfr_err
erfr_end:
	ajmp	pcstr_h



;---------------------------------------------------------;

intm:	acall	newline
	mov	r0, #0
intm2:	acall	newline
	cjne	r0, #0x80, intm3 
	ajmp	newline
intm3:	mov	a, r0
	acall	phex
	mov	a, #':'
	acall	cout
intm4:	acall	space
	mov	a, @r0
	acall	phex
	inc	r0
	mov	a, r0
	anl	a, #00001111b
	jnz	intm4
	sjmp	intm2





;**************************************************************
;**************************************************************
;*****							  *****
;*****	     2k page boundry is somewhere near here	  *****
;*****	       (no ajmp or acall past this point)	  *****
;*****							  *****
;**************************************************************
;**************************************************************



;---------------------------------------------------------;
;							  ;
;   Subroutines for memory managment and non-serial I/O	  ;
;							  ;
;---------------------------------------------------------;



	; poll the flash rom using it's toggle bit feature
	; on D6... and wait until the flash rom is not busy
	; dptr must be initialized with the address to read
flash_wait:
	push	b
	clr	a
	movc	a, @a+dptr
flwt2:	mov	b, a
	inc	r5
	clr	a
	movc	a, @a+dptr
	cjne	a, b, flwt2
	pop	b
	push	dph
	push	dpl
	mov	dptr, #dram_enable
	clr	a
	movx	@dptr, a
	pop	dpl
	pop	dph
	ret

	;send the flash enable codes
flash_en:
	mov	dptr, #dram_disable
	clr	a
	movx	@dptr, a
flash_en2:
	mov	dptr, #flash_en1_addr
	mov	a, #flash_en1_data
	movx	@dptr, a
	mov	dptr, #flash_en2_addr
	mov	a, #flash_en2_data
	movx	@dptr, a
	ret


;a routine that writes ACC to into flash memory at DPTR
; C is set if error occurs, C is clear if it worked

prgm:	xch	a, r0
	push	acc
	push	dpl
	push	dph
	lcall	flash_en		;do first step, enable writing
	mov	dptr, #flash_wr_addr
	mov	a, #flash_wr_data
	movx	@dptr, a		;send flash write command
	pop	dph
	pop	dpl
	mov	a, r0
	movx	@dptr, a		;write the data
	lcall	flash_wait		;wait until it's done
	clr	a
	movc	a, @a+dptr		;read it back
	clr	c
	xrl	a, r0
	jz	prgmend			;check if data written ok
	setb	c
prgmend:pop	acc
	xch	a, r0
	ret


	;send a custom erase command.  This is used by erall,
	;and it's intended to be callable from the flash memory
	;so that custom block erase code can be implemented
erblock:
	push	acc
	push	dpl
	push	dph
	lcall	flash_en		;send flash enable stuff
	mov	dptr, #flash_er1_addr
	mov	a, #flash_er1_data
	movx	@dptr, a		;send erase enable
	lcall	flash_en2		;send flash enable stuff
	pop	dph
	pop	dpl
	pop	acc
	movx	@dptr, a		;send erase command
	sjmp	flash_wait


; erase the entire flash rom
; C=1 if failure, C=0 if ok

erall:
	mov	dptr, #flash_er2_addr
	mov	a, #flash_er2_data
	acall	erblock			;use erblock to send erase all
	mov	dptr, #bflash
erall2:	clr	a
	movc	a, @a+dptr		;read back flash memory
	cpl	a
	jnz	erall_err		;check if it's really erased
	inc	dptr
	mov	a, #((eflash+1) & 255)
	cjne	a, dpl, erall2
	mov	a, #(((eflash+1) >> 8) & 255)
	cjne	a, dph, erall2
	clr	c
	ret
erall_err:
	setb	c
	ret





;finds the next header in the external memory.
;  Input DPTR=point to start search (only MSB used)
;  Output DPTR=location of next module
;    C=set if a header found, C=clear if no more headers
find:	mov	dpl, #0
	clr	a
	movc	a, @a+dptr
	cjne	a, #0xA5, find3
	inc	dptr
	clr	a
	movc	a, @a+dptr
	cjne	a, #0xE5, find3
	inc	dptr
	clr	a
	movc	a, @a+dptr
	cjne	a, #0xE0, find3
	inc	dptr
	clr	a
	movc	a, @a+dptr
	cjne	a, #0xA5, find3
	mov	dpl, #0			;found one here!
	setb	c
	ret
find3:	mov	a, #(emem >> 8)
	cjne	a, dph, find4		;did we just check the end
	clr	c
	ret
find4:	inc	dph			;keep on searching
	sjmp	find




;************************************
;To make PAULMON2 able to write to other
;types of memory than RAM and flash rom,
;modify this "smart_wr" routine.  This
;code doesn't accept any inputs other
;that the address (dptr) and value (acc),
;so this routine must know which types
;of memory are in what address ranges
;************************************


;Write to Flash ROM or ordinary RAM.  Carry bit will indicate
;if the value was successfully written, C=1 if not written.


smart_wr:
	push	acc
	push	b
	mov	b, a
	;there is a flash rom, but is this address in it?
	mov	a, dph
	cjne	a, #(eflash >> 8), isfl3
	sjmp	wr_flash
isfl3:	jnc	wr_ram
	cjne	a, #(bflash >> 8), isfl4
	sjmp	wr_flash
isfl4:	jnc	wr_flash
	;sjmp	wr_ram

wr_ram: mov	a, b
	movx	@dptr, a	;write the value to memory
	clr	a
	movc	a, @a+dptr	;read it back from code memory
	clr	c
	subb	a, b
	jz	smwrok
	movx	a, @dptr	;read it back from data memory
	clr	c
	subb	a, b
	jz	smwrok
smwrbad:setb	c
	sjmp	smwrxit
smwrok: clr	c
smwrxit:pop	b
	pop	acc
	ret

wr_flash:
	mov	a, b
	lcall	prgm
	pop	b
	pop	acc
	ret




;---------------------------------------------------------;
;							  ;
;	Power-On initialization code and such...	  ;
;							  ;
;---------------------------------------------------------;

;first the hardware has to get initialized.

intr_return:
	reti

poweron:

	;set low power OTP eprom mode for philips 87C52
	mov	0xA2, #255

	clr	a
	mov	ie, a		;all interrupts off
	mov	ip, a
	mov	psw, #psw_init
	;clear any interrupt status, just in case the user put
	;"ljmp 0" inside their interrupt service code.
	acall	intr_return
	acall	intr_return
	cpl	a
	mov	p0, a
	mov	p1, a
	mov	p2, a
	mov	p3, a
	mov	sp, #stack
	acall	set_bank0
	acall	rst_devices

;initialize the serial port
	mov	a, #baud_const
	mov	th1, a
	mov	tl1, a
	mov	tmod, #0x21	;set timer #1 for 8 bit auto-reload
	mov	pcon, #0x80	;configure built-in uart
	mov	scon, #0x52
	setb	tr1		;start the baud rate timer

;now print out the nice welcome message

welcome:
	mov	r0, #24
welcm2: lcall	newline
	djnz	r0, welcm2
	mov	dptr, #logon1
	lcall	pcstr
	lcall	dir

;check for the user to press "S" to avoid auto-startup.
;allow a 1/2 second timeout, and accept and 'S' that's
;already in sbuf

; 0.5 seconds is 307200 instruction cycles at 7.3728 MHz
; 307200 = 256 * 240 * 5

aabort0:
	mov	r0, #0
aabort1:
	mov	r1, #240
aabort2:
	jnb	ri, aabort3
	clr	ri
	mov	a, sbuf
	lcall	upper
	cjne	a, #'S', aabort0 ;begin another 1/2 wait if not 'S'
	sjmp	stcode5		;don't do startup code if 'S'
aabort3:nop
	djnz	r1, aabort2
	djnz	r0, aabort1


stcode: mov	dptr, #bmem	 ;search for startup routines
stcode2:lcall	find
	jnc	stcode5
	mov	dpl, #4
	clr	a
	movc	a, @a+dptr
	cjne	a, #253, stcode4	;only startup code if matches B
	push	dph
	mov	a, #(stcode3 & 255)
	push	acc
	mov	a, #(stcode3 >> 8)
	push	acc
	mov	dpl, #64
	clr	a
	jmp	@a+dptr		;jump to the startup code
stcode3:pop	dph		;hopefully it'll return to here
stcode4:inc	dph
	mov	a, dph
	cjne	a, #((emem+1) >> 8) & 255, stcode2
stcode5:			;now we've executed all of 'em


warm_boot:
	clr	a
	mov	ie, a		;all interrupts off
	mov	ip, a
	acall	intr_return
	acall	intr_return
	mov	psw, #psw_init
	mov	sp, #stack
	acall	set_bank0
	mov	r0, #250
wb_ti:	jb	ti, wb_ti2
	nop
	djnz	r0, wb_ti
wb_ti2:
	mov	a, #baud_const	;initialize the serial port
	mov	th1, a
	mov	tl1, a
	mov	tmod, #0x21	;set timer #1 for 8 bit auto-reload
	mov	pcon, #0x80	;configure built-in uart
	mov	scon, #0x52
	setb	tr1		;start the baud rate timer

	mov	r6, #(pgm & 255)
	mov	r7, #(pgm >> 8)
	ljmp	menu












;---------------------------------------------------------;
;							  ;
;     More subroutines, but less frequent used, so	  ;
;     they're down here in the second 2k page.		  ;
;							  ;
;---------------------------------------------------------;



;this twisted bit of code looks for escape sequences for
;up, down, left, right, pageup, and pagedown, as well
;as ordinary escape and ordinary characters.  Escape
;sequences are required to arrive with each character
;nearly back-to-back to the others, otherwise the characters
;are treated as ordinary user keystroaks.  cin_filter
;returns a single byte when it sees the multi-byte escape
;sequence, as shown here.

; return value	 key	      escape sequence
;   11 (^K)	 up	      1B 5B 41
;   10 (^J)	 down	      1B 5B 42
;   21 (^U)	 right	      1B 5B 43
;    8 (^H)	 left	      1B 5B 44
;   25 (^Y)	 page up      1B 5B 35 7E
;   26 (^Z)	 page down    1B 5B 36 7E

.equ	esc_char, 27

cin_filter:
	jnb	ri, cinf1
	lcall	cin
	cjne	a, #esc_char, cinf_end
	;if esc was already in sbuf, just ignore it
cinf1:	lcall	cin
	cjne	a, #esc_char, cinf_end
cinf2:	acall	cinf_wait
	jb	ri, cinf4
	mov	a, #esc_char
	ret			;an ordinary ESC

cinf4:	;if we get here, it's a control code, since a character
	;was received shortly after receiving an ESC character
	lcall	cin
	cjne	a, #'[', cinf_consume
	acall	cinf_wait
	jnb	ri, cin_filter
	lcall	cin
cinf5a: cjne	a, #'A', cinf5b
	mov	a, #11
	ret
cinf5b: cjne	a, #'B', cinf5c
	mov	a, #10
	ret
cinf5c: cjne	a, #'C', cinf5d
	mov	a, #21
	ret
cinf5d: cjne	a, #'D', cinf5e
	mov	a, #8
	ret
cinf5e: cjne	a, #0x35, cinf5f
	sjmp	cinf8
cinf5f: cjne	a, #0x36, cinf5g
	sjmp	cinf8
cinf5g: sjmp	cinf_consume		;unknown escape sequence

cinf8:	;when we get here, we've got the sequence for pageup/pagedown
	;but there's one more incoming byte to check...
	push	acc
	acall	cinf_wait
	jnb	ri, cinf_restart
	lcall	cin
	cjne	a, #0x7E, cinf_notpg
	pop	acc
	add	a, #228
cinf_end: ret
cinf_restart:
	pop	acc
	sjmp	cin_filter
cinf_notpg:
	pop	acc
;unrecognized escape... eat up everything that's left coming in
;quickly, then begin looking again
cinf_consume:
	acall	cinf_wait
	jnb	ri, cin_filter
	lcall	cin
	cjne	a, #esc_char, cinf_consume
	sjmp	cinf2

;this thing waits for a character to be received for approx
;4 character transmit time periods.  It returns immedately
;or after the entire wait time.	 It does not remove the character
;from the buffer, so ri should be checked to see if something
;actually did show up while it was waiting
	.equ	char_delay, 4		;number of char xmit times to wait
cinf_wait:
	mov	a, r2
	push	acc
	mov	r2, #char_delay*5
cinfw2: mov	a, th0
cinfw3: jb	ri, cinfw4
	inc	a
	jnz	cinfw3
	djnz	r2, cinfw2
cinfw4: pop	acc
	mov	r2, a
	ret




pint8u: ;prints the unsigned 8 bit value in Acc in base 10
	push	b
	push	acc
	sjmp	pint8b

pint8:	;prints the signed 8 bit value in Acc in base 10
	push	b
	push	acc
	jnb	acc.7, pint8b
	mov	a, #'-'
	lcall	cout
	pop	acc
	push	acc
	cpl	a
	add	a, #1
pint8b: mov	b, #100
	div	ab
	setb	f0
	jz	pint8c
	clr	f0
	add	a, #'0'
	lcall	cout
pint8c: mov	a, b
	mov	b, #10
	div	ab
	jnb	f0, pint8d
	jz	pint8e
pint8d: add	a, #'0'
	lcall	cout
pint8e: mov	a, b
	add	a, #'0'
	lcall	cout
	pop	acc
	pop	b
	ret



	;print 16 bit unsigned integer in DPTR, using base 10.
pint16u:	;warning, destroys r2, r3, r4, r5, psw.5
	push	acc
	mov	a, r0
	push	acc
	clr	psw.5
	mov	r2, dpl
	mov	r3, dph

pint16a:mov	r4, #16		;ten-thousands digit
	mov	r5, #39
	acall	pint16x
	jz	pint16b
	add	a, #'0'
	lcall	cout
	setb	psw.5

pint16b:mov	r4, #232	;thousands digit
	mov	r5, #3
	acall	pint16x
	jnz	pint16c
	jnb	psw.5, pint16d
pint16c:add	a, #'0'
	lcall	cout
	setb	psw.5

pint16d:mov	r4, #100	;hundreds digit
	mov	r5, #0
	acall	pint16x
	jnz	pint16e
	jnb	psw.5, pint16f
pint16e:add	a, #'0'
	lcall	cout
	setb	psw.5

pint16f:mov	a, r2		;tens digit
	mov	r3, b
	mov	b, #10
	div	ab
	jnz	pint16g
	jnb	psw.5, pint16h
pint16g:add	a, #'0'
	lcall	cout

pint16h:mov	a, b		;and finally the ones digit
	mov	b, r3
	add	a, #'0'
	lcall	cout

	pop	acc
	mov	r0, a
	pop	acc
	ret

;ok, it's a cpu hog and a nasty way to divide, but this code
;requires only 21 bytes!  Divides r2-r3 by r4-r5 and leaves
;quotient in r2-r3 and returns remainder in acc.  If Intel
;had made a proper divide, then this would be much easier.

pint16x:mov	r0, #0
pint16y:inc	r0
	clr	c
	mov	a, r2
	subb	a, r4
	mov	r2, a
	mov	a, r3
	subb	a, r5
	mov	r3, a
	jnc	pint16y
	dec	r0
	mov	a, r2
	add	a, r4
	mov	r2, a
	mov	a, r3
	addc	a, r5
	mov	r3, a
	mov	a, r0
	ret



;pcstr prints the compressed strings.  A dictionary of 128 words is
;stored in 4 bit packed binary format.	When pcstr finds a byte in
;a string with the high bit set, it prints the word from the dictionary.
;A few bytes have special functions and everything else prints as if
;it were an ordinary string.

; special codes for pcstr:
;    0 = end of string
;   13 = CR/LF
;   14 = CR/LF and end of string
;   31 = next word code should be capitalized

pcstr:	push	acc
	mov	a, r0
	push	acc
	mov	a, r1
	push	acc
	mov	a, r4
	push	acc
	setb	psw.1
	setb	psw.5
pcstr1: clr	a
	movc	a, @a+dptr
	inc	dptr
	jz	pcstr2
	jb	acc.7, decomp
	anl	a, #0x7F
pcstrs1:cjne	a, #13, pcstrs2
	lcall	newline
	setb	psw.1
	sjmp	pcstr1
pcstrs2:cjne	a, #31, pcstrs3
	clr	psw.5
	sjmp	pcstr1
pcstrs3:cjne	a, #14, pcstrs4
	lcall	newline
	sjmp	pcstr2
pcstrs4:
	clr	psw.1
	lcall	cout
	sjmp	pcstr1
pcstr2: pop	acc
	mov	r4, a
	pop	acc
	mov	r1, a
	pop	acc
	mov	r0, a
	pop	acc
	ret

;dcomp actually takes care of printing a word from the dictionary

; dptr = position in packed words table
; r4=0 if next nibble is low, r4=255 if next nibble is high

decomp: anl	a, #0x7F
	mov	r0, a		;r0 counts which word
	jb	psw.1, decomp1	;avoid leading space if first word
	lcall	space
decomp1:clr	psw.1
	push	dpl
	push	dph
	mov	dptr, #words
	mov	r4, #0
	mov	a, r0
	jz	dcomp3
	;here we must seek past all the words in the table
	;that come before the one we're supposed to print
	mov	r1, a
dcomp2: acall	get_next_nibble
	jnz	dcomp2
	;when we get here, a word has been skipped... keep doing
	;this until we're pointing to the correct one
	djnz	r1, dcomp2
dcomp3: ;now we're pointing to the correct word, so all we have
	;to do is print it out
	acall	get_next_nibble
	jz	dcomp_end
	cjne	a, #15, dcomp4
	;the character is one of the 12 least commonly used
	acall	get_next_nibble
	inc	a
	movc	a, @a+pc
	sjmp	dcomp5
	.db	"hfwgybxvkqjz"
dcomp4: ;the character is one of the 14 most commonly used
	inc	a
	movc	a, @a+pc
	sjmp	dcomp5
	.db	"etarnisolumpdc"
dcomp5: ;decide if it should be uppercase or lowercase
	mov	c, psw.5
	mov	acc.5, c
	setb	psw.5
	cjne	r0, #20, dcomp6
	clr	acc.5
dcomp6: cjne	r0, #12, dcomp7
	clr	acc.5
dcomp7: lcall	cout
	sjmp	dcomp3
dcomp_end:
	pop	dph
	pop	dpl
	ajmp	pcstr1

get_next_nibble:	;...and update dptr and r4, of course
	clr	a
	movc	a, @a+dptr
	cjne	r4, #0, gnn2
	mov	r4, #255
	anl	a, #00001111b
	ret
gnn2:	mov	r4, #0
	inc	dptr
	swap	a
	anl	a, #00001111b
	ret


;---------------------------------------------------------;
;							  ;
;	 Here begins the data tables and strings	  ;
;							  ;
;---------------------------------------------------------;

;this is the dictionary of 128 words used by pcstr.

words:
	.db	0x82, 0x90, 0xE8, 0x23, 0x86, 0x05, 0x4C, 0xF8
	.db	0x44, 0xB3, 0xB0, 0xB1, 0x48, 0x5F, 0xF0, 0x11
	.db	0x7F, 0xA0, 0x15, 0x7F, 0x1C, 0x2E, 0xD1, 0x40
	.db	0x5A, 0x50, 0xF1, 0x03, 0xBF, 0xBA, 0x0C, 0x2F
	.db	0x96, 0x01, 0x8D, 0x3F, 0x95, 0x38, 0x0D, 0x6F
	.db	0x5F, 0x12, 0x07, 0x71, 0x0E, 0x56, 0x2F, 0x48
	.db	0x3B, 0x62, 0x58, 0x20, 0x1F, 0x76, 0x70, 0x32
	.db	0x24, 0x40, 0xB8, 0x40, 0xE1, 0x61, 0x8F, 0x01
	.db	0x34, 0x0B, 0xCA, 0x89, 0xD3, 0xC0, 0xA3, 0xB9
	.db	0x58, 0x80, 0x04, 0xF8, 0x02, 0x85, 0x60, 0x25
	.db	0x91, 0xF0, 0x92, 0x73, 0x1F, 0x10, 0x7F, 0x12
	.db	0x54, 0x93, 0x10, 0x44, 0x48, 0x07, 0xD1, 0x26
	.db	0x56, 0x4F, 0xD0, 0xF6, 0x64, 0x72, 0xE0, 0xB8
	.db	0x3B, 0xD5, 0xF0, 0x16, 0x4F, 0x56, 0x30, 0x6F
	.db	0x48, 0x02, 0x5F, 0xA8, 0x20, 0x1F, 0x01, 0x76
	.db	0x30, 0xD5, 0x60, 0x25, 0x41, 0xA4, 0x2C, 0x60
	.db	0x05, 0x6F, 0x01, 0x3F, 0x26, 0x1F, 0x30, 0x07
	.db	0x8E, 0x1D, 0xF0, 0x63, 0x99, 0xF0, 0x42, 0xB8
	.db	0x20, 0x1F, 0x23, 0x30, 0x02, 0x7A, 0xD1, 0x60
	.db	0x2F, 0xF0, 0xF6, 0x05, 0x8F, 0x93, 0x1A, 0x50
	.db	0x28, 0xF0, 0x82, 0x04, 0x6F, 0xA3, 0x0D, 0x3F
	.db	0x1F, 0x51, 0x40, 0x23, 0x01, 0x3E, 0x05, 0x43
	.db	0x01, 0x7A, 0x01, 0x17, 0x64, 0x93, 0x30, 0x2A
	.db	0x08, 0x8C, 0x24, 0x30, 0x99, 0xB0, 0xF3, 0x19
	.db	0x60, 0x25, 0x41, 0x35, 0x09, 0x8E, 0xCB, 0x19
	.db	0x12, 0x30, 0x05, 0x1F, 0x31, 0x1D, 0x04, 0x14
	.db	0x4F, 0x76, 0x12, 0x04, 0xAB, 0x27, 0x90, 0x56
	.db	0x01, 0x2F, 0xA8, 0xD5, 0xF0, 0xAA, 0x26, 0x20
	.db	0x5F, 0x1C, 0xF0, 0xF3, 0x61, 0xFE, 0x01, 0x41
	.db	0x73, 0x01, 0x27, 0xC1, 0xC0, 0x84, 0x8F, 0xD6
	.db	0x01, 0x87, 0x70, 0x56, 0x4F, 0x19, 0x70, 0x1F
	.db	0xA8, 0xD9, 0x90, 0x76, 0x02, 0x17, 0x43, 0xFE
	.db	0x01, 0xC1, 0x84, 0x0B, 0x15, 0x7F, 0x02, 0x8B
	.db	0x14, 0x30, 0x8F, 0x63, 0x39, 0x6F, 0x19, 0xF0
	.db	0x11, 0xC9, 0x10, 0x6D, 0x02, 0x3F, 0x91, 0x09
	.db	0x7A, 0x41, 0xD0, 0xBA, 0x0C, 0x1D, 0x39, 0x5F
	.db	0x07, 0xF2, 0x11, 0x17, 0x20, 0x41, 0x6B, 0x35
	.db	0x09, 0xF7, 0x75, 0x12, 0x0B, 0xA7, 0xCC, 0x48
	.db	0x02, 0x3F, 0x64, 0x12, 0xA0, 0x0C, 0x27, 0xE3
	.db	0x9F, 0xC0, 0x14, 0x77, 0x70, 0x11, 0x40, 0x71
	.db	0x21, 0xC0, 0x68, 0x25, 0x41, 0xF0, 0x62, 0x7F
	.db	0xD1, 0xD0, 0x21, 0xE1, 0x62, 0x58, 0xB0, 0xF3
	.db	0x05, 0x1F, 0x73, 0x30, 0x77, 0xB1, 0x6F, 0x19
	.db	0xE0, 0x19, 0x43, 0xE0, 0x58, 0x2F, 0xF6, 0xA4
	.db	0x14, 0xD0, 0x23, 0x03, 0xFE, 0x31, 0xF5, 0x14
	.db	0x30, 0x99, 0xF8, 0x03, 0x3F, 0x64, 0x22, 0x51
	.db	0x60, 0x25, 0x41, 0x2F, 0xE3, 0x01, 0x56, 0x27
	.db	0x93, 0x09, 0xFE, 0x11, 0xFE, 0x79, 0xBA, 0x60
	.db	0x75, 0x42, 0xEA, 0x62, 0x58, 0xA0, 0xE5, 0x1F
	.db	0x53, 0x4F, 0xD1, 0xC0, 0xA3, 0x09, 0x42, 0x53
	.db	0xF7, 0x12, 0x04, 0x62, 0x1B, 0x30, 0xF5, 0x05
	.db	0xF7, 0x69, 0x0C, 0x35, 0x1B, 0x70, 0x82, 0x2F
	.db	0x2F, 0x14, 0x4F, 0x51, 0xC0, 0x64, 0x25, 0x00

;STR

logon1: .db	"Welcome",128,148,"2 v2.1 (MP3 Player Version)",13
	.db	"by",31,248,31,254,13,13
	.db	"http://www.pjrc.com/tech/mp3/",13,13
	.db	31,228," 'S'",128,160,188,143,13,14
abort:	.db	" ",31,158,31,160,"!",13,14
prompt1:.db	148,"2_MP3 Loc:",0
prompt2:.db	" >", 160	;must follow after prompt1
prompt3:.db	134,202,130,'(',0
prompt4:.db	"),",149,140,128,200,": ",0
prompt5:.db	31,151,130,195,"s",199,166,131,","
	.db	186," JUMP",128,134,161,"r",130,13,14
prompt6:.db	13,13,31,135,131,129,": ",0
prompt7:.db	31,228,251," key: ",0
prompt8:.db	13,13,31,136,128,131,129," (",0
prompt9:.db	13,13,31,130,31,253,0
prompt9b:.db	 31,129,32,32,32,32,32,31,201,14	;must follow prompt9
prompt10:.db	") ",31,135,31,178,": ",0
beg_str:.db	"First",31,129,": ",0
end_str:.db	"Last",31,129,":",32,32,0
sure:	.db	31,185,161," sure?",0
edits1: .db	13,13,31,156,154,146,",",140,128,200,14
edits2: .db	"  ",31,156,193,",",142,129,247,13,14
dnlds1: .db	13,13,31,159," ascii",249,150,31,152,132,137
	.db	",",149,140,128,160,13,14
dnlds2: .db	13,31,138,160,"ed",13,14
dnlds3: .db	13,31,138,193,"d",13,14
dnlds4: .db	"Summary:",14
dnlds5: .db	" ",198,"s",145,"d",14
dnlds6a:.db	" ",139,145,"d",14
dnlds6b:.db	" ",139," written",14
dnlds7: .db	31,155,":",14
dnlds8: .db	" ",139," unable",128," write",14
dnlds9: .db	32,32,"bad",245,"s",14
dnlds10:.db	" ",133,159,150,198,14
dnlds11:.db	" ",133,132,157,14
dnlds12:.db	" ",133," non",132,157,14
dnlds13:.db	31,151,155," detected",13,14
runs1:	.db	13,134,"ning",130,":",13,14
uplds3: .db	13,13,"Sending",31,152,132,137,172,32,32,0
uplds4: .db	" ",128,32,32,0		;must follow uplds3
help1txt:.db	13,13,"Standard",31,158,"s",14
help2txt:.db	31,218,31,244,"ed",31,158,"s",14
type1:	.db	31,154,158,0
type2:	.db	31,130,0
type4:	.db	31,143,31,226,31,170,0
type5:	.db	"???",0
help_cmd2:.db	31,215,0
help_cmd: .db	31,142,215,209,0	;these 11 _cmd string must be in order
dir_cmd:  .db	31,209,130,"s",0
run_cmd:  .db	31,134,130,0
dnld_cmd: .db	31,138,0
upld_cmd: .db	31,147,0
nloc_cmd: .db	31,135,129,0
jump_cmd: .db	31,136,128,131,129,0
dump_cmd: .db	31,132,219,154,131,0
intm_cmd: .db	31,132,219,192,131,0
edit_cmd: .db	31,156,154,146,0
clrm_cmd: .db	31,237,131,0
erfr_cmd: .db	31,203,153,144,0
erfr_ok:  .db	31,153,144,203,'d',13,14
erfr_err: .db	31,133,155,13,14





;-----------------------------------------------------------
;  MP3 player specific functions
;-----------------------------------------------------------


	.org	0x0FC0

;to call a function in either bank, code should call here
;with the function's address in DPTR, and the bank in carry.
;This allows either bank to call an ordinary callable function
;is either bank, but it is slow and requires 5 bytes of stack
;space instead of the usual 2 bytes

call_function:
	push	selected_bank_byte	; remember which bank we're in now
	mov	flash_a16_pin, c	; switch to the target bank
	mov	selected_bank_bit, c
	mov	a, #(ret_function & 255) ;when the function returns, return
	push	acc			; to "ret_function" so we can restore
	mov	a, #(ret_function >> 8) ; the selected bank and properly
	push	acc			; return to the caller
	clr	a
	jmp	@a+dptr			; run the function

rst_devices:
	clr	reset_pin
	mov	r0, #200
	djnz	r0, *
	setb	reset_pin
	ret

ret_function:
	pop	selected_bank_byte
	mov	c, selected_bank_bit
	mov	flash_a16_pin, c
	ret

	.org	0x0FE0

;use this do call library functions in bank1 from bank0, where
;the library function is coded specifically to return properly
;This allows library functions located in bank1 to be called
;from the main program, located in bank 0, with minimal overhead.
;The library functions have to be coded to JMP to "ret_bank1_lib".

call_bank1_lib:
	setb	selected_bank_bit
	setb	flash_a16_pin
	clr	a
	jmp	@a+dptr			; run the function


	;set to bank 1
set_bank1:
	setb	c
set_bank:
	mov	selected_bank_bit, c
	mov	flash_a16_pin, c
	ret


;library functions in bank1, intended to be called from bank0
;will jump here to return to their caller

	.org	0x0FF0

ret_bank1_lib:
set_bank0:
	clr	selected_bank_bit
	clr	flash_a16_pin
	ret


	;which bank is currently selected?
get_bank:
	mov	c, selected_bank_bit
	ret











;---------------------------------------------------------;
;                                                         ;
;                  External Memory Editor                 ;
;                                                         ;
;---------------------------------------------------------;

;register usage:
; R4,    Flags:
;         bit0: 0=display CODE memory, 1=display DATA memory
;         bit1: 0=editing disabled, 1=editing enabled
;         bit2: 0=editing in hex, 1=editing in ascii
;         bit3: 0=normal, 1=in middle of hex entry (value in r5)
; R6/R7, current memory location
;

.org	0x1000
.db	0xA5,0xE5,0xE0,0xA5	;signiture
.db	254,'E',0,0		;id (254=user installed command)
.db	0,0,0,0			;prompt code vector
.dB	0,0,0,0			;reserved
.db	0,0,0,0			;reserved
.db	0,0,0,0			;reserved
.db	0,0,0,0			;user defined
.db	255,255,255,255		;length and checksum (255=unused)
.db	"Memory Editor (VT100)",0

.org	0x1040            ;executable code begins here


	mov	r4, #0
	acall	redraw
main:
	mov	a, r4
	clr	acc.3
	mov	r4, a
main2:	lcall   cin_filter
	acall	input_ck_2nd
cmd1:	cjne	a, #27, cmd2		;quit
	ajmp	quit
cmd2:	cjne	a, #11, cmd3		;up
	ajmp	cmd_up
cmd3:	cjne	a, #10, cmd4		;down
	ajmp	cmd_down
cmd4:	cjne	a, #8, cmd5		;left
	ajmp	cmd_left
cmd5:	cjne	a, #21, cmd6		;right
	ajmp	cmd_right
cmd6:	cjne	a, #12, cmd7		;redraw
	acall	redraw
	ajmp	main
cmd7:	cjne	a, #17, cmd8		;quit
	ajmp	quit
cmd8:	cjne	a, #3, cmd9		;code memory
	mov	a, r4
	anl	a, #11111110b
	mov	r4, a
	acall	cursor_home
	mov	dptr, #str_code
	acall	pstr_hh
	acall	redraw_data
	ajmp	main
cmd9:	cjne	a, #4, cmd10		;data memory
	mov	a, r4
	orl	a, #00000001b
	mov	r4, a
	acall	cursor_home
	mov	dptr, #str_data
	acall	pstr_hh
	acall	redraw_data
	ajmp	main
cmd10:	cjne	a, #7, cmd11		;goto memory loc
	ajmp	cmd_goto
cmd11:	cjne	a, #5, cmd12		;toggle editing
	ajmp	cmd_edit
cmd12:	cjne	a, #6, cmd13		;fill memory
	ajmp	cmd_fill
cmd13:	cjne	a, #1, cmd14		;edit in ascii
	mov	a, r4
	jnb	acc.1, main
	setb	acc.2
	mov	r4, a
	acall	erase_commands
	acall	print_commands
	acall	redraw_cursor
	ajmp	main
cmd14:	cjne	a, #24, cmd15		;edit in hex
        mov     a, r4
        jnb     acc.1, main
        clr     acc.2
        mov     r4, a
        acall   erase_commands
        acall   print_commands
        acall   redraw_cursor
        ajmp    main
cmd15:	cjne    a, #25, cmd16           ;page up
	ajmp	cmd_pgup
cmd16:	cjne    a, #26, cmd17           ;page down
	ajmp	cmd_pgdown
cmd17:


cmd_data:	;the input character wasn't any particular command, so
		;maybe it's some input data being typed for edit mode
	mov	b, a			;keep a copy of user data in b
	mov	a, r4
	jb	acc.1, cmd_data2
cmd_abort:
	ajmp	main			;ignore if not in edit mode
cmd_data2:
	jnb	acc.2, input_hex
input_ascii:
	mov	a, b
	acall	ascii_only
	cjne	a, b, cmd_abort		;check that input is an ascii char
	mov	dph, r7
	mov	dpl, r6
	acall	edit_smart_wr		;write the char to memory
	ajmp	cmd_right

input_hex:
	mov	a, b
	lcall	upper
	lcall	asc2hex
	jc	cmd_abort		;ignore if not hex
	mov	r0, a			;keep hex value of input in r0
        mov     dph, r7			;load dptr with address
        mov     dpl, r6
	mov	a, r4
	jb	acc.3, input_hex_2nd
input_hex_1st:
	mov	a, r0
	mov	r5, a
	mov	a, r4
	setb	acc.3		;remember that we're waiting for 2nd char
	mov	r4, a
	acall	redraw_cursor
	ajmp	main2
input_hex_2nd:
	mov	a, r5			;get data from memory
	swap	a			;shift nibbles
	anl	a, #11110000b		;just in case
	add	a, r0			;add in this input to lower part
	acall	edit_smart_wr		;write back to memory
	mov	a, r4
	clr	acc.3
	mov	r4, a
	ajmp	cmd_right

input_ck_2nd:
	;the main input routine will always call here when a new
	;byte is entered... so we can do something special if we
	;were waiting for the second character and it is not a
	;legal hex character
	push	acc
	mov	a, r4
	jb	acc.1, inck2d
	;if editing is disabled, don't do anything
	clr	acc.3
inck2b:	mov	r4, a
inck2c:	pop	acc
	ret
inck2d:	jnb	acc.3, inck2b
	;if we get here, we were actually waiting for the 2nd char
	pop	acc
	push	acc
	lcall	upper
	lcall	asc2hex
	jnc	inck2c		;proceed normally if it is valid
	;if we get here, we did not get a hex legal char
	pop	acc
	push	acc
	cjne	a, #esc_char, inck2e
        mov     a, r4
        clr     acc.3
        mov     r4, a
        acall   redraw_cursor
	pop	acc
	pop	acc		;don't return and do the quit cmd
	pop	acc		;just quit this entry and wait for next cmd
	ajmp	main
inck2e: mov     dph, r7                 ;load dptr with address
        mov     dpl, r6
	mov	a, r5
        acall   edit_smart_wr                ;write to memory
	mov	a, r4
	clr	acc.3
	mov	r4, a
	acall	redraw_cursor
	sjmp	inck2c

; R4,    Flags:
;         bit0: 0=display CODE memory, 1=display DATA memory
;         bit1: 0=editing disabled, 1=editing enabled
;         bit2: 0=editing in hex, 1=editing in ascii
;         bit3: 0=normal, 1=in middle of hex entry (value in r5)


cmd_fill:
	mov	a, r4
	anl	a, #00000010b
	jnz	cmd_fill_ok
	ajmp	main			;don't allow if not in editing mode
cmd_fill_ok:
	acall	erase_commands
        mov     a, r4
        push    acc
	mov	dptr, #fill_prompt1
	acall	pcstr_hh
	lcall	ghex16
	jc	cmd_fill_abort
	jb	psw.5, cmd_fill_abort
	mov	r0, dpl
	mov	r1, dph
	mov	dptr, #fill_prompt2
	acall	pstr_hh
	lcall	ghex16
	jc	cmd_fill_abort
	jb	psw.5, cmd_fill_abort
	mov	r4, dpl
	mov	r5, dph
	mov	dptr, #fill_prompt3
	acall	pcstr_hh
	lcall	ghex
	jc	cmd_fill_abort
	jb	psw.5, cmd_fill_abort
	mov	r2, a
	mov	a, r4
	mov	r6, a
	mov	a, r5
	mov	r7, a
	pop	acc
	mov	r4, a
	mov	dpl, r0
	mov	dph, r1
	;now r4 is restored to its normal value, dptr holds the
	;first location to fill, and r6/r7 holds the last location to
	;fill, and r2 has the fill value.
cmd_fill_loop:
	mov	a, r2
	acall	edit_smart_wr
	mov	a, r6
	cjne	a, dpl, cmd_fill_next
	mov	a, r7
	cjne	a, dph, cmd_fill_next
	;when we get here, we're done!
	acall	erase_commands
	acall	print_commands
	acall	redraw_data
	ajmp	main
cmd_fill_next:
	inc	dptr
	sjmp	cmd_fill_loop

cmd_fill_abort:
        pop     acc
        mov     r4, a
        acall   erase_commands
        acall   print_commands
	acall	redraw_cursor
	ajmp	main

fill_prompt1:
        .db     "Fill",31,131,"; First: ",0
fill_prompt2:
        .db     "  Last: ",0
fill_prompt3:
        .db     " ",168,": ",0 

cmd_edit:
	acall	erase_commands
	mov	a, r4
	xrl	a, #00000010b
	mov	r4, a
        acall   print_commands
        acall   redraw_cursor
        ajmp    main

cmd_goto:
	acall	erase_commands
	mov	dptr, #goto_prompt
	acall	pcstr_hh
	mov	a, r4
	push	acc
	lcall	ghex16
	pop	acc
	mov	r4, a
	jc	cmdgt_abort
	jb	psw.5, cmdgt_abort
        mov     r6, dpl
        mov     r7, dph
        acall   cursor_home
        mov     a, #20
        acall   cursor_down
	acall	print_commands
	acall	redraw_data
	ajmp	main
cmdgt_abort:
        acall   cursor_home
        mov     a, #20
        acall   cursor_down
	acall	print_commands
	acall	redraw_cursor
	ajmp	main


goto_prompt:
	.db	31,131,31,129,": ",0

cmd_up:
	acall	blank_it
        mov     a, r6
        clr     c
        subb    a, #16
        mov     r6, a
        mov     a, r7
        subb    a, #0
        mov     r7, a
	mov	a, r6
	cpl	a
	anl	a, #11110000b
	jz	cmd_up_scroll
        mov     a, #1
        acall   cursor_up
        acall   invert_it
        ajmp    main
cmd_up_scroll:
	acall	redraw_data
	ajmp	main

cmd_pgup:
	dec	r7
	acall	redraw_data
	ajmp	main

cmd_pgdown:
	inc	r7
	acall	redraw_data
	ajmp	main

cmd_down:
	acall	blank_it
        mov     a, r6
        add     a, #16
        mov     r6, a
        mov     a, r7
        addc    a, #0
        mov     r7, a
	mov	a, r6
	anl	a, #11110000b
	jz	cmd_down_scroll
	mov	a, #1
	acall	cursor_down
	acall	invert_it
	ajmp	main
cmd_down_scroll:
	acall	redraw_data
	ajmp	main


cmd_left:
	acall	blank_it
	mov	a, #3
	acall	cursor_left
	mov	a, r6
	clr	c
	subb	a, #1
	mov	r6, a
	mov	a, r7
	subb	a, #0
	mov	r7, a
	mov	a, r6
	orl	a, #11110000b
	cpl	a
	jz	cmdlf2
	acall	invert_it
	ajmp	main
cmdlf2:
	mov	a, r6
	cpl	a
	anl	a, #11110000b
	jz	cmd_left_scroll
	mov	a, #48
	acall	cursor_right
	mov	a, #1
	acall	cursor_up
	acall	invert_it
	ajmp	main
cmd_left_scroll:
	acall	redraw_data
	ajmp	main


cmd_right:
	acall	blank_it
	mov	a, #3
	acall	cursor_right
	mov	dpl, r6
	mov	dph, r7
	inc	dptr
	mov	r6, dpl
	mov	r7, dph
	mov	a, r6
	anl	a, #00001111b
	jz	cmdrt2
	acall	invert_it
	ajmp	main

cmdrt2:
	mov	a, r6
	anl	a, #11110000b
	jz	cmd_right_scroll
	mov	a, #48
	acall	cursor_left
	mov	a, #1
	acall	cursor_down
	acall	invert_it
	ajmp	main
cmd_right_scroll:
	acall	redraw_data
	ajmp	main




;register usage:
; R4,    Flags:
;         bit0: 0=display CODE memory, 1=display DATA memory
;         bit1: 0=editing disabled, 1=editing enabled
;         bit2: 0=editing in hex, 1=editing in ascii
;	  bit3: 0=normal, 1=in middle of hex entry (value in r5)
; R6/R7, current memory location
;


redraw:
        mov     dptr, #str_cl		;clear screen
        acall   pstr_hh
	acall	print_title_line
	acall	newline_hh
	acall	print_addr_line
	acall	newline_hh
	acall	print_dash_line
        acall   newline_hh
	mov	a, #16
	acall	cursor_down
        acall   print_dash_line
        acall   newline_hh
        acall   print_commands
redraw_data:
	acall	cursor_home
	mov	a, #2
	acall	cursor_down
	;compute first byte address to display on the screen
	mov	dpl, #0
	mov	dph, r7
	;now display the data
	mov	r0, #16
rd2:	acall	newline_hh
	lcall	phex16
	mov	a, #':'
	acall	cout_hh
	mov	r2, dpl
	mov	r3, dph
rd3:	acall	space_hh
	acall	read_dptr
	acall	phex_hh
	inc	dptr
	mov	a, dpl
	anl	a, #00001111b
	jnz	rd3
	mov	dpl, r2
	mov	dph, r3
	acall	space_hh
	acall	space_hh
	acall	space_hh
rd4:	acall	read_dptr
	acall	ascii_only
	acall	cout_hh
	inc	dptr
	mov	a, dpl
	anl	a, #00001111b
	jnz	rd4
	djnz	r0, rd2

redraw_cursor:
	acall	cursor_home
	mov	a, r6
	swap	a
	anl	a, #00001111b
	add	a, #3
	acall	cursor_down
	;make the ascii character inverse
	mov	a, r6
	anl	a, #00001111b
	add	a, #56
	acall	cursor_right
	acall	inverse_on
	acall	read_r6r7
	acall	ascii_only
	acall	cout_hh
	acall	inverse_off

	;now make the hex value inverse
	mov	a, r6
	anl	a, #00001111b
	rl	a
	cpl	a
	add	a, #52
	acall	cursor_left
	acall	inverse_on
	acall	read_r6r7
	acall	phex_hh
	ajmp	inverse_off


blank_it:
	mov	a, r6
	anl	a, #00001111b
	rl	a
	cpl	a
	add	a, #49
	acall	cursor_right
	acall	read_r6r7
	acall	ascii_only
	acall	cout_hh
	mov	a, r6
	anl	a, #00001111b
	rl	a
	cpl	a
	add	a, #52
	acall	cursor_left
	acall	read_r6r7
	ajmp	phex_hh

invert_it:
        mov     a, r6
        anl     a, #00001111b
        rl      a
        cpl     a
        add     a, #49
        acall   cursor_right
        acall   read_r6r7
        acall   ascii_only
	acall	inverse_on
        acall   cout_hh
	acall	inverse_off
        mov     a, r6
        anl     a, #00001111b
        rl      a
        cpl     a
        add     a, #52
        acall   cursor_left
        acall   read_r6r7
	acall	inverse_on
        acall   phex_hh
	ajmp	inverse_off






quit:	mov	a, r6
	anl	a, #11110000b
	swap	a
	cpl	a
	add	a, #19
	acall	cursor_down
	ajmp	newline_hh


ascii_only:
        anl     a, #01111111b   ;avoid unprintable characters
	cjne	a, #127, aonly2
	mov	a, #' '
	ret
aonly2: clr     c
        subb    a, #32
        jnc     aonly3          ;avoid control characters
        mov     a, #(' ' - 32)
aonly3: add     a, #32
	ret





read_dptr:
	mov	a, r4
	jb	acc.0, rddptr2
	clr	a
	movc	a, @a+dptr
	ret
rddptr2:movx	a, @dptr
	ret

read_r6r7:
	push	dph
	push	dpl
	mov	dph, r7
	mov	dpl, r6
	mov	a, r4
	jb	acc.3, rdr6r7d
	jb	acc.0, rdr6r7b
	clr	a
	movc	a, @a+dptr
	sjmp	rdr6r7c
rdr6r7b:movx	a, @dptr
rdr6r7c:pop	dpl
	pop	dph
	ret
rdr6r7d:mov	a, r5
	sjmp	rdr6r7c


cursor_home:
	acall	term_esc
	mov	a, #'H'
	ajmp	cout_hh

cursor_down:	;acc is # of lines to move down
	acall	term_esc
	acall	pint_hh
	mov	a, #'B'
	ajmp	cout_hh

cursor_up:	;acc is # of lines to move up
	acall	term_esc
	acall	pint_hh
	mov	a, #'A'
	ajmp	cout_hh

cursor_left:	;acc is # of characters to move left
	acall	term_esc
	acall	pint_hh
	mov	a, #'D'
	ajmp	cout_hh

cursor_right:	;acc is # of characters to move right
	acall	term_esc
	acall	pint_hh
	mov	a, #'C'
	ajmp	cout_hh

inverse_on:
	mov	dptr, #str_so
	ajmp	pstr_hh

str_so:	.db	esc_char, "[0;7m", 0

inverse_off:
	mov	dptr, #str_se
	ajmp	pstr_hh

str_se:	.db	esc_char, "[0m", 0


term_esc:
	push	acc
	mov	a, #esc_char
	acall	cout_hh
	mov	a, #'['
	acall	cout_hh
	pop	acc
	ret
	
print_addr_line:
	mov	dptr, #str_addr
	acall	pstr_hh
	mov	r0, #0
paddrl: acall	space_hh
	mov	a, #'+'
	acall	cout_hh
	mov	a, r0
	lcall	phex1
	inc	r0
	cjne	r0, #16, paddrl
	mov	dptr, #str_ascii_equiv
	ajmp	pstr_hh


print_dash_line:
	mov	r0, #72
pdashl: mov	a, #'-'
	acall	cout_hh
	djnz	r0, pdashl
	ret

print_title_line:
	mov	a, r4
	jb	acc.0, ptitle2
	mov	dptr, #str_code
	sjmp	ptitle3
ptitle2:mov	dptr, #str_data
ptitle3:acall	pstr_hh
	mov	r0, #8
ptitlel:acall	space_hh
	djnz	r0, ptitlel
	mov	dptr, #str_title
	ajmp	pcstr_hh

erase_commands:
        acall   cursor_home
        mov     a, #20
        acall   cursor_down
        mov     r2, #72
ercmd2: acall   space_hh
        djnz    r2, ercmd2
	mov	a, #72
	ajmp	cursor_left


; R4,    Flags:
;         bit0: 0=display CODE memory, 1=display DATA memory
;         bit1: 0=editing disabled, 1=editing enabled
;         bit2: 0=editing in hex, 1=editing in ascii
;         bit3: 0=normal, 1=in middle of hex entry (value in r5)

print_commands:
        mov     a, r4
        jnb     acc.1, pcmd_no_edit
	mov	dptr, #str_cmd3
	jb	acc.2, pcmd_ascii
	mov	dptr, #str_cmd4
pcmd_ascii:
	acall	pstr_hh
	mov	dptr, #str_cmd5
	acall	pstr_hh
	sjmp	pcmd_finish
pcmd_no_edit:
	mov     dptr, #str_cmd2
	acall   pstr_hh
pcmd_finish:
        mov     dptr, #str_cmd1
        ajmp    pstr_hh



edit_smart_wr:
	mov	a, r4
	jnb	acc.0, edit_sw2

	push	b
	mov	b, a
	movx	@dptr, a	;write the value to memory
	nop
	movx	a, @dptr	;read it back from data memory
	clr	c
	subb	a, b
	jz	esmwrok
	pop	b
	setb	c
	ret
esmwrok:pop	b
	clr	c
	ret
edit_sw2:
	ljmp	smart_wr



str_cmd1: .db "  ^G=Goto  ^C=Code  ^D=Data  ^L=Redraw  ^Q=Quit", 0
str_cmd2: .db "^E-Edit",0
str_cmd3: .db "^A=", esc_char, "[0;7m", "ASCII", esc_char, "[0m", "  ^X=Hex", 0
str_cmd4: .db "^A=ASCII  ^X=", esc_char, "[0;7m", "Hex", esc_char, "[0m", 0
str_cmd5: .db "  ^F=Fill",0


str_cl:	.db	esc_char, "[H", esc_char, "[2J", 0

str_addr: .db "ADDR:",0
str_ascii_equiv: .db	"   ASCII EQUIVILANT",0
str_title: .db	"8051",31,154,31,131,31,216,"or,",31,248,31,254,", 1996",0
str_code: .db "CODE",0
str_data: .db "DATA",0


cout_hh:ljmp	cout
phex_hh:ljmp	phex
pstr_hh:ljmp	pstr
newline_hh:ljmp	newline
pcstr_hh:ljmp	pcstr
pint_hh:ljmp	pint8u
space_hh:ljmp	space



